/*
 * Copyright 2016 - 2025 Gary Talent (gary@drinkingtea.net). All rights reserved.
 */

#include <studio/imguiutil.hpp>

#include <studio/filepickerpopup.hpp>

namespace studio {

FilePickerPopup::FilePickerPopup(
	ox::StringParam name,
	keel::Context &kctx,
	ox::StringParam fileExt) noexcept:
	m_name{std::move(name)},
	m_explorer{kctx},
	m_fileExts{std::move(fileExt)} {
}

FilePickerPopup::FilePickerPopup(
	ox::StringParam name,
	keel::Context &kctx,
	ox::Vector<ox::String> fileExts) noexcept:
	m_name{std::move(name)},
	m_explorer{kctx},
	m_fileExts{std::move(fileExts)} {
}

void FilePickerPopup::refresh() noexcept {
	m_explorer.setModel(buildFileTreeModel(
		m_explorer,
		[this](ox::StringViewCR path, ox::FileStat const &s) {
			auto const [ext, err] = fileExt(path);
			return
				s.fileType == ox::FileType::Directory ||
				(s.fileType == ox::FileType::NormalFile && !err && m_fileExts.contains(ext));
		},
		false).or_value(ox::UPtr<FileTreeModel>{}));
}

void FilePickerPopup::open() noexcept {
	refresh();
	m_open = true;
}

void FilePickerPopup::close() noexcept {
	m_explorer.setModel(ox::UPtr<FileTreeModel>{});
	m_open = false;
}

bool FilePickerPopup::isOpen() const noexcept {
	return m_open;
}

ox::Optional<ox::String> FilePickerPopup::draw(StudioContext &ctx) noexcept {
	ox::Optional<ox::String> out;
	if (!m_open) {
		return out;
	}
	if (ig::BeginPopup(ctx.tctx, m_name, m_open, {380, 340})) {
		auto const vp = ImGui::GetContentRegionAvail();
		m_explorer.draw(ctx, {vp.x, vp.y - 30});
		if (ig::PopupControlsOkCancel(m_open) == ig::PopupResponse::OK) {
			auto p = m_explorer.selectedPath();
			if (p) {
				out.emplace(*p);
			}
			close();
		}
		ImGui::EndPopup();
	}
	return out;
}

}
